// |$Id: genwaw.c,v 1.1.1.1 2022/12/27 22:06:24 pm Exp $          |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://173.255.241.228              | 
   | FUNCTION NAME: genwaw.c                                    |
   | DIRECTORY: bsu-3.0.2/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: setting geometry for a Walk-a-Way shooting        |
   |          Typical use might be for SASW or MASW shooting    |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2019 Paul Michaels                          |
   |  <paulmichaels@boisestate.edu>                             |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |------------------------------------------------------------|
   |                                                            |
   |This version compiles both with gcc and MingW               |
   |Problem was scanning directory                              |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <stddef.h>

      #include <time.h>
      #include <dirent.h>
      #include <errno.h>
      #include "../include/c_bsegy.h"
      
      #define FMAX 20
      #define PREC 16

//...parameter definitions

//...function prototypes
      #include "../include/sub4.h"


      void outlst(int Nsources, int Nchanl, 
        char pid[], char lstfile[], FILE *h3);

      void writenez(int indx, float y, float x, float z, char ID[], FILE *h4);
	
      char**  GetFileNames(char **mybuffer, int *Nsources, char filterUC[], 
	char filterLC[]);

	int cstring_cmp(const void *a, const void *b);

      int  main(int argc, char *argv[])
     {

//...declare variables
      FILE              *h1, *h2, *h3, *h4;
      int               jt,
                        jrec,
                        npts,
                        ntraces,
                        ibar,
                        invbar;
      float             fsamin,
                        bar,
                        *s1,
                        *s2;
      char              pid[5]="waw",   //process id name (4char)
                        ofile[13],
                        lstfile[13]="genwaw.lst",
			nezfile[]="WAW000.nez",
                        infile[14];
      struct header     hd;

//hardwire for 100 files
#define MAX 100

//...specific to this application
      	int             Nsources;
	int		Nchanl;
	int		indx;
	int		kjrec;
	int		krec;
	int		j;
	short		gazi[8],gver[8];
	char		Kname[9];
	char		cmd[40];
	char		ID[12];
	char		*buffer[MAX];
	char		filterUC[]="DAT";
	char		filterLC[]="dat";
	float		sx,sy,sz;
	float		rx,ry,rz;

//...open output listing file
        if( (h3=fopen(lstfile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",lstfile);
          goto quick_exit;
        }


/* FILE INFORMATION  */
	fprintf(stdout,"\nEnter Number of Channels ");
	scanf("%d",&Nchanl);
	printf("GEOPHONE ORIENTATIONS\n");
	printf("Geophone Az 90=East Ver 180=Down \n");
	for (j=0; j<Nchanl; j++) {
	printf("Channel=%1.1d Enter Geophone Orientation  Az  Ver\n",j);
	scanf("%d %d",&gazi[j],&gver[j]);
	} //next j channel

/*
//...open NEZ file
        if( (h4=fopen(nezfile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",nezfile);
          goto quick_exit;
        }
*/
	indx = 0;
	GetFileNames(buffer,&Nsources,filterUC,filterLC);

	outlst(Nsources,Nchanl,pid,lstfile,h3);

	printf("Nsources= %d \n",Nsources);
	printf("\n");

/* FILE LOOP*/
//	sprintf(cmd,"rm -r -f LST");
//	system(cmd);
	#ifdef WIN
	sprintf(cmd,"mkdir LST");
	system(cmd);
	#else
	sprintf(cmd,"mkdir -p LST");
	system(cmd);
	#endif
	for ( jrec = 0; jrec<Nsources; jrec ++) {
	printf("%s \n",buffer[Nsources-jrec-1]);
	#ifdef WIN
	sprintf(cmd,"egg2seg %s ",buffer[Nsources-jrec-1]);
	#else
	sprintf(cmd,"egg2seg %s >/dev/null",buffer[Nsources-jrec-1]);
	#endif	
	system(cmd);
	strncpy(Kname,buffer[Nsources-jrec-1],4);
	kjrec = atoi(Kname);
	#ifdef WIN
	sprintf(cmd,"move %4.4d.lst LST",kjrec);
	system(cmd); 
	#else
	sprintf(cmd,"mv %4.4d.lst LST",kjrec);
	system(cmd); 
	#endif
//...open input file
	sprintf(infile,"%4.4d.seg",kjrec);
        if( (h1=fopen(infile,"rb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",infile);
          goto exit_3;
        }

//...open output file
	sprintf(ofile,"gwaw%4.4d.seg",kjrec);
        if( (h2=fopen(ofile,"wb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",ofile);
          goto exit_1;
        }

//...check input file size
          if(in_chk(&ntraces,&npts,&fsamin,hd,h1)!=0)
         {
          goto exit_2;
         }

//...print input file data to lstfile
	fprintf(h3,"\n     File = %s  ", buffer[Nsources-jrec-1]);
        fprintf(h3,"\n     Number of samples= %d",npts);
        fprintf(h3,"\n     Sample Interval = %5.4f sec.",fsamin);
        fprintf(h3,"\n     Number of traces = %d",ntraces);
fflush(h3);
	if (indx == 0) {
//...dynamic allocation of seimic trace memory
        s1=calloc(npts,sizeof(float));
          if(s1==NULL)
         {
          fprintf(stderr,"ABORT:s1 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_2;
         }
        s2=calloc(npts,sizeof(float));
          if(s2==NULL)
         {
          fprintf(stderr,"ABORT:s2 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_s1;
         }
	} //endif
	indx = indx + 1;

//-----------------main traces loop---------------------------
      for (krec=0;krec<ntraces;krec++)
     {

//...read trace
        if(c_bsegin(krec,s1,npts,&hd,h1)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegin");
        goto quit_it;
       }

//...interactive querry shot coordinates
//	printf("\n smul1=%d smul2=%d ishlo=%d ishla=%d irlo=%d irla%d ",
//		hd.smul1,hd.smul2,hd.ishlo,hd.ishla,hd.irlo,hd.irla);
	if (krec == 0 ) {
/*
	printf("\n Sx=%d Sy=%d Sz=%d \n ",
		hd.ishlo,hd.ishla,hd.ishe);
*/
	printf("SOURCE LOCATION-------------------------------------");
	printf("\n %s: Enter Source X  Y  Z \n",buffer[Nsources-jrec-1]);
	scanf("%f %f %f",&sx,&sy,&sz);
	hd.smul1=-100;  //elevation scale factor
	hd.smul2=-100;  // x, y scale factor (Ex. if hd.irlo=950, Rx=9.50 m)
	hd.ishlo=round(sx*100.);
	hd.ishla=round(sy*100.);
	hd.ishe= round(sz*100.);
	} //endif first trace, enter shot coord once
	hd.smul1=-100;  //elevation scale factor
	hd.smul2=-100;  // x, y scale factor (Ex. if hd.irlo=950, Rx=9.50 m)
/*
	printf("\n Rx=%d Ry=%d Rz=%d \n ",
		hd.irlo,hd.irla,hd.irel);
*/
	printf("\n Trace:%2.2d Enter Receiver X  Y  Z \n",krec);
	scanf("%f %f %f",&rx,&ry,&rz);
	hd.irlo=round(rx*100.);
	hd.irla=round(ry*100.);
	hd.irel=round(rz*100.);

// saved geophone orientations for this channel krec
	hd.geoazi = gazi[krec];
	hd.geover = gver[krec];	
		
//...output trace
        if(c_bsegout(krec,s1,npts,&hd,h2)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegout");
        goto quit_it;
       }
	}  //next krec traces loop
	fclose(h1);
	fclose(h2);

//... move edited gwawxxxx.seg over xxxx.seg
	sprintf(cmd,"mv gwaw%4.4d.seg %4.4d.seg",kjrec,kjrec);
	system(cmd);

} //next jrec shot file loop
	free(s1);
	free(s2);
        return(0);
quit_it:
        puts(" ");
        free(s2);
exit_s1:
        free(s1);
exit_2:
//        fclose(h2);
exit_1:
//        fclose(h1);
exit_3:
        fclose(h3);
//	fclose(h4);
quick_exit:
        return(1);
     } //  end of main
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <paulmichaels@boisestate.edu>          |
   | FUNCTION NAME: outlst.c                                    |
   | Copyright (C) 2021 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: prints copyright header, file names, and          |
   |          echo check of input parameters in newly           |
   |          opened output listing file.                       |
   |                                                            |
   |------------------------------------------------------------|
*/
      void outlst(int Nsources, int Nchanl, 
        char pid[], char lstfile[], FILE *h3)
     {
	int k;

//...print copyright label to outlst and screen

      fcr_labl(pid,h3);

//...echo check
	fputs("\n",h3);                      //blank line
	fprintf(h3,"\n     PROGRAM: gen%s.c ",pid);
	fputs("\n",h3);                      //blank line
	fputs("\n     FILES:       ",h3);
	fprintf(h3,"\n     Listing File= %s",lstfile);
	fputs("\n",h3);                      //blank line
	fputs("\n     INPUT PARAMETERS: ",h3);
	fprintf(h3,"\n     Number Sources=\t %5.5d ",Nsources);
	fprintf(h3,"\n     Number Channels=\t %5.5d ",Nchanl);
	fputs("\n",h3);                      //blank line
     }  //  end outlst

//===============================================================================
	void writenez(int indx, float y, float x, float z, char ID[], FILE *h4)
	{
	fprintf(h4,"%11.6d %11.4f %11.4f %11.4f %s \n",indx,y,x,z,ID); 
	} //end writenez	
// Following works with both gcc and MingW
//===============================================================================

	char**  GetFileNames(char **mybuffer, int *Nsources, char filterUC[],
	char filterLC[])
	{
	DIR *dir;
	struct dirent *entry;
	int k=0;

	if ((dir = opendir(".")) == NULL)
	perror("opendir() error");
	  else {
	   while  ((entry = readdir(dir)) != NULL) {
	    if (strstr( (entry->d_name),filterUC) || 
		strstr((entry->d_name),filterLC)  ) {
 //    	    printf("%s\n", entry->d_name);
	    mybuffer[k] =malloc(strlen(entry->d_name)+1);
	    strcpy(mybuffer[k],(entry->d_name));
	    k=k+1;
	    }
	   }
    	closedir(dir);
	 }
	
	*Nsources = k;
	if (*Nsources > MAX) {
	fprintf(stderr,"ABORT: More files than MAX in directory\n");
	} //endif

	qsort (mybuffer, *Nsources, sizeof(mybuffer[0]), cstring_cmp);

	return mybuffer;
	} //end GetFileNames

int cstring_cmp(const void *a, const void *b) 
{ 
    const char **ia = (const char **)a;
    const char **ib = (const char **)b;
// b befor a, sorts ascending, a before b descending on return
    return strcmp(*ib, *ia);
	/* strcmp functions works exactly as expected from
	comparison function */ 
} 
/*  The following works with gcc, but not MingW 
//===============================================================================

	char**  GetFileNames(char **mybuffer, int *Nsources, char filter[])
	{
	int	k=0;
	struct dirent **namelist;
	int n;

	n = scandir(".", &namelist, NULL, alphasort);
	if (n == -1) {
	perror("scandir");
	exit(EXIT_FAILURE);
	}  //endif
// while decrements n until zero
	while (n -- ) {
	if (strstr( (namelist[n]->d_name),filter)) {
	mybuffer[k]=(namelist[n]->d_name);
	k=k+1;
	}
	  free(namelist[n]);
  } //endwhile
	free(namelist);

	*Nsources = k;
	return mybuffer;
	} //end GetFileNames
*/

