c |$Id: bsegout.f,v 1.1.1.1 2022/12/27 22:06:24 pm Exp $          |
c#|------------------------------------------------------------|
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://173.255.241.228                | 
c | PROGRAM NAME: bsegout.f                                    |
c | DIRECTORY: bsu-3.0.3/src/sublibF4                          |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2021 Paul Michaels                          |
c |  <paulmichaels@boisestate.edu>                             |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c | bsegout: sets up header and outputs a trace in segy format |
c |                                                            |
c |  traces=vector with data                                   |
c |  npts= number of values in traces to be output             |
c |  jrec= record number for this trace                        |
c |  outfil = output file name (opened on first call of segout)|
c |  fsamin=sample rate of data                                |
c |   note: main program specifies icall                       |
c |   set icall = 0 on first call                              |
c |------------------------------------------------------------|
c
      SUBROUTINE BSEGOUT(iunit,
     +icall,npts,traces,fsamin,outfil,jrec)
      include '../Fort/include/bsegy.inc'
      real*4       traces(*)
      integer*4    npts
      integer*4    nlen
      character*40 outfil
c
      common /header/ hd
c
c     int(A,2) same as int2(A), 2=kind parameter
c     length=int(npts,2)
c
c     For large data sets with more than 32768 samples
c     Cast 4 byte npts to unsigned 2 byte length
      call usignont(npts,length)
c//////////////////////////////////////////////////////////////////
c...non segy convention for large sample rates (integer*2 isi) ////
c//// segy convention is for seismic data sample rates in      ////
c//// microseconds.  problem is that an integer*2 variable was ////
c//// chosen for this variable.  fix up here is to use a seldom////
c//// needed variable, duse, as a switch to scale the sample   ////
c//// rate to other ''units''.                                 ////
c//////////////////////////////////////////////////////////////////
c
c
c open file with the correct record length
c if is first call
      nlen=240+4*npts  
      if(icall.eq.0) then
      open (iunit,file=outfil,access='direct',form='unformatted',
     +recl=nlen)
      icall=1
      endif
c
c...output trace
      write(iunit,rec=jrec)
     +(hd(j),j=1,240),
     +(traces(j),j=1,npts)
c
      return
      end
